#include <qvbox.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qapplication.h>
#include <iostream.h>

#include "MapEditor.h"

void read_vol(const char *vol_file, double *width, double *gridx, double *gridy, double *gridz, int *extx, int *exty, int *extz, double **phi) {
	unsigned long nvox, count;
	double dgridx, dgridy, dgridz, dwidth;
	double phitest, dtemp;
	FILE *fin;
   
	fin = fopen(vol_file, "r");
	if( fin == NULL ) {
		fprintf(stderr, "lib_vio> Error: Can't open file! %s  [e.c. 13010]\n", vol_file); 
		exit(13010);   
	}

	/* read header and print information */
	fscanf(fin, "%le %le %le %le %d %d %d", &dwidth, &dgridx, &dgridy, &dgridz, extx, exty, extz);
	*width = dwidth; *gridx = dgridx; *gridy = dgridy; *gridz = dgridz;
	printf ("lib_vio> File %s - Header information: \n", vol_file);
	printf ("lib_vio> Columns, rows, and sections: x=1-%d, y=1-%d, z=1-%d\n",*extx,*exty,*extz);
	printf ("lib_vio> 3D coordinates of first voxel (1,1,1): (%f,%f,%f)\n",*gridx,*gridy,*gridz);  
	printf ("lib_vio> Voxel size in Angstrom: %f \n", *width);
	nvox = *extx * *exty * *extz;

	/* allocate memory and read data */
	printf ("lib_vio> Reading density data... \n");
	*phi = (double *) malloc(nvox * sizeof(double));
	if (*phi == NULL) {
		fprintf(stderr, "lib_vio> Error: Unable to satisfy memory allocation request [e.c. 13020]\n"); 
		exit(13020);
	}

	for (count=0;count<nvox;count++) {
		if (fscanf(fin,"%le", &dtemp) != 1) {
			fprintf(stderr, "lib_vio> Error: file %s is too short (not compatible with header information) or data unreadable [e.c. 13030]\n", vol_file); 
			exit(13030);
		} else *(*phi+count) = dtemp;
	}
	if (fscanf(fin,"%le", &phitest) != EOF) {
		fprintf(stderr, "lib_vio> Error: file %s is too long, not compatible with header information [e.c. 13040]\n", vol_file); 
		exit(13040);
	}
	fclose(fin);  
	printf ("lib_vio> Volumetric data read from file %s\n", vol_file); 
}

int main(int argc, char *argv[]) {
	QString s;
	svt_volume<double> *volumericMap;
	double *pData;
    	int iSizeX;
    	int iSizeY;
    	int iSizeZ;
	double gridx, gridy, gridz, width;
	
	QApplication app(argc, argv);
	
	if (argc < 2) {
		s = QFileDialog::getOpenFileName(".", "Maps (*.situs)", NULL, "open file dialog", "Choose a file");  
		if (s == NULL) return 0; 	
  	} else s = argv[1];
	
	
    	
	read_vol(s, &width, &gridx, &gridy, &gridz, &iSizeX, &iSizeY, &iSizeZ, &pData);
	volumericMap = new svt_volume<double>();
	volumericMap->setData(iSizeX, iSizeY, iSizeZ, pData);
	MapEditor *mapEditor = new MapEditor(volumericMap);   
    app.setMainWidget(mapEditor);
    mapEditor->show();
    return app.exec();
}
